@@ -213,8 +213,10 @@ function (x,
         }
 
     if (type < 3 && !is.null(class.weights)) {
-        if (is.character(class.weights) && class.weights == "inverse")
+        if (is.character(class.weights) && class.weights == "inverse") {
             class.weights <- 1 / table(y)
+            names(class.weights) = lev
+        }
         if (is.null(names(class.weights)))
             stop("Weights have to be specified along with their according level names !")
         weightlabels <- match (names(class.weights), lev)
